*$include MaritimeModel10P6V-1
*$include MaritimeModelToy-1
*$include MaritimeModelP2C4V3-1
$include MaritimeModelP4C2V3-1

file CPLEX_Options / CPLEX.opt / ;
  put CPLEX_Options
  put
    'nodefileind 3'    /

    'workmem 500'

    ;
  putclose CPLEX_Options ;


alias(n,m);
alias(t,tt,ttt);

Binary variables
op(v,n,t)        ship v operate on port n at time t
of(v,n,t)        ship v prepare to operate on port n at time t
wa(v,n,t)        ship v wait on port n at time t
sa(v,n,m,t)      ship v sail from port n to m at time t
;

Positive variables
st(n,t)          stock of port n at the beginning of time period t
la(v,t)          load of ship v at the beginning of time period t
sla1(v,n,m,t)    auxilary variable for bilinear constraints
sla2(v,n,m,t)    auxilary variable for bilinear constraints

stf(n,t)         stock of port n at the end of period
;

Variables
TotalCost        objective variable
;

Parameter
RProdd(p)        production rate change to discrete time
RConsd(d)        consumption rate change to discrete time
qd(v,p)          loading rate change to discrete time
dd(v,d)          discharge rate change to discrete time
TimeWUpd(v)      time upperbound of ship v for waiting change to discrete time
tfd(v,n)         preparing time for operation of ship v at port n
Csailfd(v)       fixed cost of ship v sailing
Csailvd(v)       variable cost of ship v sailing
Cwaitd(v,n)      unit waiting cost of ship v at port n
Coperd(v,n)      unit operation cost of ship v at port n
Coperfd(v,n)     unit preparing cost for operation of ship v at port n
distd(n,m)        distance between the two ports in time
;
RProdd(p)= RProd(p)*TimePeriod/card(t);
RConsd(d)= RCons(d)*TimePeriod/card(t);
qd(v,p)=TimePeriod/(tv(v,p)*card(t));
dd(v,d)=TimePeriod/(tv(v,d)*card(t));
TimeWUpd(v)=TimeWUp(v)*card(t)/TimePeriod;
tfd(v,n)=tf(v,n)*card(t)/TimePeriod;
Csailfd(v)=Csailf(v)*TimePeriod/card(t);
Csailvd(v)=Csailv(v)*TimePeriod/card(t);
Cwaitd(v,n)=Cwait(v,n)*TimePeriod/card(t);
Coperd(v,n)=Coper(v,n)*TimePeriod/card(t);
Coperfd(v,n)=Coper(v,n)*TimePeriod/card(t);
distd(n,m)=dist(n,m)*card(t)/TimePeriod;
distd(n,m)$(ori(n)and (p(m)or d(m))and distd(n,m)eq 0)=1;

display
RProdd
RConsd
qd
dd
TimeWUpd
tfd
Csailfd
Csailvd
Cwaitd
Coperd
Coperfd
distd
;

st.fx(n,'t1')=StIni(n);
la.fx(v,'t1')=0;

display st.l;

equation
Routing(v,n,m,t)     ship v can only sail on the required routing
Onsail(v,n,m)        ship v sail for the whole distance
Departure(v)         any ship v will departure from the departure port
Arrive(v)            any ship v will arrive at the destination port
PortCapacity(n,t)    no more than the largest number of ships can operation on port n at time t
ShipflowBala1(v,n,t)  ship flow keep balance on port n at time t
ShipflowBala2(v,n,t)  ship flow keep balance on port n at time t
NoWaitAftOp(v,n,t)   any ship v can not wait after operation
LeaveAftOp(v,n,t)    any ship v can not leave from port n before it finish the operation
NoPrepAftOp(v,n,t)   any ship v can not prepare for operation after operation
StockBalap(p,t)      stock balance of port p at time t
StockBalad(d,t)      stock balance of port d at time t
LoadBalance(v,t)     load banlance of ship v at time t
StockUpB(n,t)        stock upper bound of port n at time t
StockLoB(n,t)        stock lower bound of port n at time t
StockUpBpf(p,t)      stock upper bound of port p at final time
StockLoBpf(p,t)      stock lower bound of port p at final time
StockUpBdf(d,t)      stock upper bound of port d at final time
StockLoBdf(d,t)      stock lower bound of port d at final time
LoadUpB(v,t)         load upper bound of ship v at time t
LoadUpBf(v,t)        load upper bound of ship v at final time
TwaitUpB(v,n)        time upper bound of ship v waiting at port n
TimePreOp(v,n)       preparing time for operation of ship v at port n
SalaTrans1(v,n,m,t)  auxilary equation for bilinear constraints
SalaTrans2(v,n,m,t)  auxilary equation for bilinear constraints
SalaTrans3(v,n,m,t)  auxilary equation for bilinear constraints
CostFunc             objective function
;

Routing(v,n,m,t)..
         sa(v,n,m,t) =l= x(v,n,m);
Onsail(v,n,m)..
         sum(t,sa(v,n,m,t)) =e= distd(n,m)*x(v,n,m);
Departure(v)..
         sum((m,n,t)$(ori(m)and(p(n)or d(n)or des(n))),sa(v,m,n,t)) =e= 1;
Arrive(v)..
         sum((n,m,t)$(des(m)and(p(n)or d(n)or ori(n))and (ord(t)le card(t)-distd(n,m))),sa(v,n,m,t)) =e= 1;
PortCapacity(n,t)$(p(n)or d(n))..
         sum(v,op(v,n,t)+of(v,n,t)) =l= CapP(n);
ShipflowBala1(v,n,t)$(p(n)or d(n))..
         sum((m,tt)$((p(m)or d(m)or ori(m))and (ord(tt)eq ord(t)-distd(m,n))),sa(v,m,n,tt))
         +sum(ttt$(ord(ttt) eq ord(t)-1),wa(v,n,ttt))
         =e=
         of(v,n,t)+wa(v,n,t);
ShipflowBala2(v,n,t)$(p(n)or d(n))..
         sum(ttt$(ord(ttt) eq ord(t)-1),op(v,n,ttt)+of(v,n,ttt))
         =e=
         sum(m$(p(m)or d(m)or des(m)),sa(v,n,m,t))
         +op(v,n,t);
NoWaitAftOp(v,n,t)..
         sum(tt$(ord(tt) eq ord(t)-1),op(v,n,tt)+of(v,n,tt))
         =l=
         op(v,n,t)+of(v,n,t)+sum(m$(p(m)or d(m)or des(m)),sa(v,n,m,t));
LeaveAftOp(v,n,t)$(p(n)or d(n))..
         sum(tt$(ord(tt) eq ord(t)-1),op(v,n,tt)) =g= sum(m$(p(m)or d(m)or des(m)),sa(v,n,m,t));
NoPrepAftOp(v,n,t)..
         sum(tt$(ord(tt) ge ord(t)+1),of(v,n,tt))+op(v,n,t) =l= 1;
StockBalap(p,t)$(ord(t)ne card(t))..
         st(p,t)-sum(v,qd(v,p)*op(v,p,t)) =e= sum(tt$(ord(tt) eq ord(t)+1),st(p,tt))-RProdd(p);
StockBalad(d,t)$(ord(t)ne card(t))..
         st(d,t)+sum(v,dd(v,d)*op(v,d,t)) =e= sum(tt$(ord(tt) eq ord(t)+1),st(d,tt))+RConsd(d);
LoadBalance(v,t)$(ord(t)ne card(t))..
         la(v,t)+sum(p,qd(v,p)*op(v,p,t))-sum(d,dd(v,d)*op(v,d,t)) =e= sum(tt$(ord(tt) eq ord(t)+1),la(v,tt));
StockUpBpf(p,t)$(ord(t)eq card(t))..
         st(p,t)-sum(v,qd(v,p)*op(v,p,t))+RProdd(p) =l= StockUp(p);
StockLoBpf(p,t)$(ord(t)eq card(t))..
         st(p,t)-sum(v,qd(v,p)*op(v,p,t))+RProdd(p) =g= StockLo(p);
StockUpBdf(d,t)$(ord(t)eq card(t))..
         st(d,t)+sum(v,dd(v,d)*op(v,d,t))-RConsd(d) =l= StockUp(d);
StockLoBdf(d,t)$(ord(t)eq card(t))..
         st(d,t)+sum(v,dd(v,d)*op(v,d,t))-RConsd(d) =g= StockLo(d);
StockUpB(n,t)..
         st(n,t) =l= StockUp(n);
StockLoB(n,t)..
         st(n,t) =g= StockLo(n);
LoadUpB(v,t)..
         la(v,t) =l= laUp(v);
LoadUpBf(v,t)$(ord(t)eq card(t))..
         la(v,t)+sum(p,qd(v,p)*op(v,p,t))-sum(d,dd(v,d)*op(v,d,t)) =l= laUp(v);
TwaitUpB(v,n)..
         sum(t,wa(v,n,t)) =l= TimeWUpd(v);
TimePreOp(v,n)..
         sum(t,of(v,n,t)) =e= tfd(v,n)*sum(m,x(v,m,n));
SalaTrans1(v,n,m,t)..
         sla1(v,n,m,t)+sla2(v,n,m,t) =e= la(v,t);
SalaTrans2(v,n,m,t)..
         sla1(v,n,m,t) =l= sa(v,n,m,t)*laUp(v);
SalaTrans3(v,n,m,t)..
         sla2(v,n,m,t) =l= (1-sa(v,n,m,t))*laUp(v);
CostFunc..
         TotalCost =e=
                 sum((v,n,m,t),sa(v,n,m,t)*Csailfd(v)*distd(n,m)+Csailvd(v)*sla1(v,n,m,t)*distd(n,m))
                 +sum((v,n,t),Cwaitd(v,n)*wa(v,n,t))
                 +sum((v,n,t),Coperfd(v,n)*of(v,n,t)+Coperd(v,n)*op(v,n,t))
                 ;

model SC
/
Routing
*Onsail
Departure
Arrive
PortCapacity
ShipflowBala1
ShipflowBala2
NoWaitAftOp
LeaveAftOp
NoPrepAftOp
StockBalap
StockBalad
LoadBalance
StockUpB
StockLoB
StockUpBpf
StockLoBpf
StockUpBdf
StockLoBdf
LoadUpB
LoadUpBf
TwaitUpB
TimePreOp
SalaTrans1
SalaTrans2
SalaTrans3
$ontext
$offtext
CostFunc
/;

SC.reslim= 30000;

Solve SC using mip minimizing TotalCost;

stf.l(d,t)$(ord(t)eq card(t))=st.l(d,t)+sum(v,dd(v,d)*op.l(v,d,t))-RConsd(d);
stf.l(p,t)$(ord(t)eq card(t))=st.l(p,t)-sum(v,qd(v,p)*op.l(v,p,t))+RProdd(p);

display
op.l
of.l
wa.l
sa.l
st.l
stf.l
la.l
sla1.l
*sla2.l
TotalCost.l
;
