*$include MaritimeModel10P6V-1
*$include MaritimeModelToy-1
*$include MaritimeModelP2C4V3-1
$include MaritimeModelP4C2V3-1

alias(l,ll,lll);
binary variables
ys(v,n,l)  ship v starts operation at event point l
ye(v,n,l)  ship v ends operation at event point l
yse(v,n,l) auxilary variable for ys and ye
yf(v,n,l)  ship v finishes preparing for operation
;

positive variables
Tep(n,l)  time of event point l of port n

to(v,n)    time ship v starts to operation on port n
tfi(v,n)    time ship v finishes preparing for operation
ta(v,n)    time ship v arrives at port n
tw(v,n)    time duration ship v waiting for at port n
tl(v,n)    time ship v leaves form port n

qi(v,p)    the amount of ship v load at port p
di(v,d)    the amount of ship v discharge at port d
st(n,l)    stock of port n at time poit l
la1(v,n)    load of ship v at port n before operation
la2(v,n)    load of ship v at port n after operation

yft(v,n,l,ll)  auxiliary variable for bilinear terms
yft1(v,n,l,ll) auxiliary variable for bilinear terms
yet(v,n,l,ll)  auxiliary variable for bilinear terms
yet1(v,n,l,ll) auxiliary variable for bilinear terms

yfq(v,p,l)   auxiliary variable for bilinear terms
yfq1(v,p,l)  auxiliary variable for bilinear terms
yfd(v,d,l)   auxiliary variable for bilinear terms
yfd1(v,d,l)  auxiliary variable for bilinear terms

stf(n)     the final stock at the end of schedule horizon
;

variables
TotalCost  total cost of the shipping;

*ta.fx(v,'n1')=0;
*ships arrive virtual port n1 at the beginning of the schedule horizon
*tl.fx('v2','n1')=5.2;

la1.fx(v,'n1')=0;
la2.fx(v,'n1')=0;
*ships load are 0 at the beginning of the schedule period

Tep.fx(n,l)$(ord(l)gt Nep(n))=Timeperiod;

alias(n,m);

scalar BigM1, BigM2;
BigM1=TimePeriod;

equation
TepCons(n,l)      time of event point can not exceed timeperiod
SeqTep(n,l,ll)    sequence constraints of time of event point
Ship2port(v,n)    ship v visit n
ShipE2S(v,n)      ship v will ends operation if it starts
Ship2porto(v,n)    ship v visit virtual origin
ShipE2So(v,n)      ship v will ends operation if it starts at virtual origin
StEtEn(v,n,l)     ship v must end its operation after it starts
Ship2portf(v,n)   ship v will prepare if it starts

TimeConss1(v,n,l) ship v starts operation on port n at event point l
TimeConse1(v,n,l) ship v ends operation on port n at event point l
TimeConsf1(v,n,l) ship v finishes preparing for operation on port n at event point l
TimeConss2(v,n,l) ship v starts operation on port n at event point l
TimeConse2(v,n,l) ship v ends operation on port n at event point l
TimeConsf2(v,n,l) ship v finishes preparing for operation on port n at event point l

NomoreShip(n,l)    number of ships can not exceed its capacity operating at any port at the same time

ArrvTime(v,n)      the time ship v arrive port n
OperStTime(v,n)    the time constraint between operation starting and arrival
OperFixTime(v,n)   the time constraint between operation starting and finish preparing
WaitTime(v,n)      waiting time of ship v at port n
OperTimeV0(v,n)    operation time of ship v at virtual ports
OperTimep(v,p)     operation time of ship v at port p
OperTimed(v,d)     operation time of ship v at port d
OperTimeV1(v,n)     operation time of ship v at virtual ports
OperTimeV2(v,n)     operation time of ship v at virtual ports
WaitUpperBound(v,n) upper bound of ship v's waiting time
LeavTimeVD(v,n)    leaving time of ship v from virtual destination port

StockBalap(p,l,ll)    stock balance of port p at event point l
StockBalad(d,l,ll)    stock balance of port d at event point l

StockBalap0(p,l)   stock balance of port p at time period k1
StockBalad0(d,l)   stock balance of port d at time period k1
StockBalapf1(p)  stock upperbound of port p at the final time period
StockBaladf1(d)  stock upperbound of port d at the final time period
StockBalapf2(p)  stock lowerbound of port p at the final time period
StockBaladf2(d)  stock lowerbound of port d at the final time period
StockUpB(n,l)      stock upperbound of ports n
StockLoB(n,l)      stock lowerbound of ports n

LoadBalap1(v,p)     load balance of ship v at port p
LoadBalad1(v,d)     load balance of ship v at port d
LoadBalap2(v,p)     load balance of ship v at port p
LoadBalad2(v,d)     load balance of ship v at port d
LoadUpB(v,p)       Upper bound of ship v's load after operation on port p
*LoadLoB(v,n)       Lower bound of ship v's load after operation on port n

YfTeptrans1(v,n,l,ll)    transition of bilinear terms
YfTeptrans2(v,n,l,ll)    transition of bilinear terms
YfTeptrans3(v,n,l,ll)    transition of bilinear terms
YeTeptrans1(v,n,l,ll)    transition of bilinear terms
YeTeptrans2(v,n,l,ll)    transition of bilinear terms
YeTeptrans3(v,n,l,ll)    transition of bilinear terms

Yfqtrans1(v,p,l)    transition of bilinear terms
Yfqtrans2(v,p,l)    transition of bilinear terms
Yfqtrans3(v,p,l)    transition of bilinear terms

Yfdtrans1(v,d,l)    transition of bilinear terms
Yfdtrans2(v,d,l)    transition of bilinear terms
Yfdtrans3(v,d,l)    transition of bilinear terms

Ysetrans1(v,n,l)    transition of bilinear terms
Ysetrans2(v,n,l)    transition of bilinear terms
Ysetrans3(v,n,l)    transition of bilinear terms

OFcost             total cost
;

TepCons(n,l)$(ord(l)le Nep(n))..
        Tep(n,l) =l= TimePeriod;
SeqTep(n,l,ll)$((ord(l) gt ord(ll))and (ord(l)le Nep(n)))..
       Tep(n,l) =g= Tep(n,ll);
Ship2port(v,n)$(not ori(n))..
        sum(l$(ord(l)le Nep(n)),ys(v,n,l)) =e= sum(m,x(v,m,n));
Ship2porto(v,n)$(ori(n))..
        sum(l$(ord(l)le Nep(n)),ys(v,n,l)) =e= 1;
ShipE2S(v,n)$(not ori(n))..
        sum(l$(ord(l)le Nep(n)),ye(v,n,l)) =e= sum(m,x(v,m,n));
ShipE2So(v,n)$(ori(n))..
        sum(l$(ord(l)le Nep(n)),ye(v,n,l)) =e= 1;
Ship2portf(v,n)..
        sum(l$(ord(l)le Nep(n)),yf(v,n,l)) =e= sum(l$(ord(l)le Nep(n)),ys(v,n,l));
StEtEn(v,n,l)$(ord(l)le Nep(n))..
        sum(ll$(ord(ll)le ord(l)),yf(v,n,ll)) =g= sum(ll$(ord(ll)le ord(l)),ye(v,n,ll));
TimeConss1(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- to(v,n) =g= -BigM1*(1-ys(v,n,l));
TimeConss2(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- to(v,n) =l= BigM1*(1-ys(v,n,l));
TimeConse1(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- tl(v,n) =g= -BigM1*(1-ye(v,n,l));
TimeConse2(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- tl(v,n) =l= BigM1*(1-ye(v,n,l));
TimeConsf1(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- tfi(v,n) =g= -BigM1*(1-yf(v,n,l));
TimeConsf2(v,n,l)$(ord(l)le Nep(n))..
        Tep(n,l)- tfi(v,n) =l= BigM1*(1-yf(v,n,l));

NomoreShip(n,l)$(ord(l)le Nep(n))..
        sum((v,ll)$(ord(ll)le ord(l)),ys(v,n,ll))-sum((v,ll)$(ord(ll)le ord(l)),ye(v,n,ll)) =l= CapP(n);

ArrvTime(v,n)$(not ori(n))..
        ta(v,n) =e= sum(m,x(v,m,n)*(tl(v,m)+dist(m,n)));
OperStTime(v,n)..
        to(v,n) =g= ta(v,n);
OperFixTime(v,n)..
        tfi(v,n) =e= to(v,n)+sum(m,x(v,m,n)*tf(v,n));
WaitTime(v,n)..
        tw(v,n) =e= to(v,n)-ta(v,n);
OperTimeV0(v,n)$(ori(n))..
        tl(v,n) =e= tfi(v,n);
OperTimep(v,p)..
        tl(v,p) =e= tfi(v,p)+sum(m,x(v,m,p)*tv(v,p)*qi(v,p));
OperTimed(v,d)..
        tl(v,d) =e= tfi(v,d)+sum(m,x(v,m,d)*tv(v,d)*di(v,d));
OperTimeV1(v,n)$(des(n))..
        tl(v,n) =e= to(v,n);
OperTimeV2(v,n)$(des(n))..
        to(v,n) =e= ta(v,n);
WaitUpperBound(v,n)$(not ori(n))..
        tw(v,n) =l= TimeWUp(v);
LeavTimeVD(v,n)$(des(n))..
        ta(v,n) =l= TimePeriod;

StockBalap(p,l,ll)$(ord(ll) eq ord(l)+1 and (ord(ll)le Nep(p)))..
        st(p,l)-sum((v,lll)$(ord(lll)le ord(l)),(yft(v,p,lll,ll)-yet(v,p,lll,ll)-yft(v,p,lll,l)+yet(v,p,lll,l))/tv(v,p))
                 +RProd(p)*(Tep(p,ll)-Tep(p,l))
           =e= st(p,ll);
StockBalad(d,l,ll)$(ord(ll) eq ord(l)+1 and (ord(ll)le Nep(d)))..
        st(d,l)+sum((v,lll)$(ord(lll)le ord(l)),(yft(v,d,lll,ll)-yet(v,d,lll,ll)-yft(v,d,lll,l)+yet(v,d,lll,l))/tv(v,d))
                 -RCons(d)*(Tep(d,ll)-Tep(d,l))
           =e= st(d,ll);

YfTeptrans1(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        yft(v,n,l,ll) =l= Timeperiod*yf(v,n,l);
YfTeptrans2(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        yft1(v,n,l,ll) =l= Timeperiod*(1-yf(v,n,l));
YfTeptrans3(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        Tep(n,ll) =e= yft(v,n,l,ll)+yft1(v,n,l,ll);
YeTeptrans1(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        yet(v,n,l,ll) =l= Timeperiod*ye(v,n,l);
YeTeptrans2(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        yet1(v,n,l,ll) =l= Timeperiod*(1-ye(v,n,l));
YeTeptrans3(v,n,l,ll)$(ord(l)le Nep(n)and (ord(ll)le Nep(n)))..
        Tep(n,ll) =e= yet(v,n,l,ll)+yet1(v,n,l,ll);

StockBalap0(p,'l1')$(Nep(p) gt 0)..
        st(p,'l1') =e= StIni(p)+RProd(p)*Tep(p,'l1');
StockBalad0(d,'l1')$(Nep(d) gt 0)..
        st(d,'l1') =e= StIni(d)-RCons(d)*Tep(d,'l1');
StockBalapf1(p)$(Nep(p) gt 0)..
        sum(l$(ord(l)eq Nep(p)),st(p,l)+RProd(p)*(TimePeriod-Tep(p,l)))
           =l= StockUp(p);
StockBalapf2(p)$(Nep(p) gt 0)..
        sum(l$(ord(l)eq Nep(p)),st(p,l)+RProd(p)*(TimePeriod-Tep(p,l)))
           =g= StockLo(p);
StockBaladf1(d)$(Nep(d) gt 0)..
        sum(l$(ord(l)eq Nep(d)),st(d,l)-RCons(d)*(TimePeriod-Tep(d,l)))
           =l= StockUp(d);
StockBaladf2(d)$(Nep(d) gt 0)..
        sum(l$(ord(l)eq Nep(d)),st(d,l)-RCons(d)*(TimePeriod-Tep(d,l)))
           =g= StockLo(d);
StockUpB(n,l)$(ord(l)le Nep(n))..
        st(n,l) =l= StockUp(n);
StockLoB(n,l)$(ord(l)le Nep(n))..
        st(n,l) =g= StockLo(n);

LoadBalap1(v,p)..
        la2(v,p) =e= sum(l$(ord(l)le Nep(p)),yfq(v,p,l))+la1(v,p);
LoadBalad1(v,d)..
        la2(v,d) =e= -sum(l$(ord(l)le Nep(d)),yfd(v,d,l))+la1(v,d);
LoadBalap2(v,p)..
        la1(v,p) =e= sum(m,x(v,m,p)*la2(v,m));
LoadBalad2(v,d)..
        la1(v,d) =e= sum(m,x(v,m,d)*la2(v,m));
LoadUpB(v,p)..
        la2(v,p) =l= laUp(v);

Yfqtrans1(v,p,l)$(ord(l)le Nep(p))..
        yfq(v,p,l) =l= laUp(v)*yf(v,p,l);
Yfqtrans2(v,p,l)$(ord(l)le Nep(p))..
        yfq1(v,p,l) =l= laUp(v)*(1-yf(v,p,l));
Yfqtrans3(v,p,l)$(ord(l)le Nep(p))..
        qi(v,p) =e= yfq(v,p,l)+yfq1(v,p,l);

Yfdtrans1(v,d,l)$(ord(l)le Nep(d))..
        yfd(v,d,l) =l= laUp(v)*yf(v,d,l);
Yfdtrans2(v,d,l)$(ord(l)le Nep(d))..
        yfd1(v,d,l) =l= laUp(v)*(1-yf(v,d,l));
Yfdtrans3(v,d,l)$(ord(l)le Nep(d))..
        di(v,d) =e= yfd(v,d,l)+yfd1(v,d,l);

Ysetrans1(v,n,l)$(ord(l)le Nep(n))..
        yse(v,n,l) =l= ys(v,n,l);
Ysetrans2(v,n,l)$(ord(l)le Nep(n))..
        yse(v,n,l) =l= 1-ye(v,n,l);
Ysetrans3(v,n,l)$(ord(l)le Nep(n))..
        yse(v,n,l) =g= ys(v,n,l)-ye(v,n,l);

OFcost..
        TotalCost =e=
                 sum(v,
                    sum((n,m), x(v,n,m)*(Csailf(v)+Csailv(v)*la2(v,n))*dist(n,m))
                  + sum(n,Cwait(v,n)*tw(v,n))
                  + sum(p,Coper(v,p)*(tl(v,p)-to(v,p)))
                  + sum(d,Coper(v,d)*(tl(v,d)-to(v,d)))
                     );

scalar
costs
costw
costo
;

model SC
/
TepCons
SeqTep
Ship2port
Ship2porto
ShipE2S
ShipE2So
*StEtEn
Ship2portf

TimeConss1
TimeConse1
TimeConsf1
TimeConss2
TimeConse2
TimeConsf2

NomoreShip

ArrvTime
OperStTime
OperFixTime
WaitTime
OperTimeV0
OperTimep
OperTimed
OperTimeV1
OperTimeV2
WaitUpperBound
LeavTimeVD

StockBalap
StockBalad

StockBalap0
StockBalad0
StockBalapf1
StockBaladf1
StockBalapf2
StockBaladf2
StockUpB
StockLoB

LoadBalap1
LoadBalad1
LoadBalap2
LoadBalad2
LoadUpB

YfTeptrans1
YfTeptrans2
YfTeptrans3
YeTeptrans1
YeTeptrans2
YeTeptrans3

Yfqtrans1
Yfqtrans2
Yfqtrans3

Yfdtrans1
Yfdtrans2
Yfdtrans3

*Ysetrans1
*Ysetrans2
*Ysetrans3
$ontext
$offtext
OFcost
/;

OPTION SYSOUT = ON;
option limrow = 100;
SC.reslim= 30000;
SC.optcr = 0.00000001;

*option MIP = Xpress;

Solve SC using mip minimizing TotalCost;

costs=sum(v,sum((n,m), x(v,n,m)*(Csailf(v)+Csailv(v)*la2.l(v,n))*dist(n,m)));
costw=sum(v,sum(n,Cwait(v,n)*tw.l(v,n)));
costo=sum(v,sum(p,Coper(v,p)*(tl.l(v,p)-to.l(v,p)))
                  + sum(d,Coper(v,d)*(tl.l(v,d)-to.l(v,d)))
                     );

stf.l(p)=sum(l$(ord(l)eq Nep(p)),st.l(p,l)+RProd(p)*(TimePeriod-Tep.l(p,l)));
stf.l(d)=sum(l$(ord(l)eq Nep(d)),st.l(d,l)-RCons(d)*(TimePeriod-Tep.l(d,l)));
display
Tep.l
ys.l,
ye.l,
yf.l
ta.l,
to.l,
tfi.l,
tw.l,
tl.l,
dist,
qi.l,
di.l,
st.l,
stf.l,
la1.l,
la2.l
yfq.l,
yfd.l,


TotalCost.l,
costs,
costw,
costo
;
