*$include MaritimeModel10P6V-1
*$include MaritimeModelToy-1
*$include MaritimeModelP2C4V3-1
$include MaritimeModelP4C2V3-1

binary variables
w(v,n,k)  whether ship v starts to operation on port n at the beginning of time slot kn
*y(n,k)    relax variable for time slot constraints
;

positive variables
y(n,k)    relax variable for time slot constraints
to(v,n)    time ship v starts to operation on port n
ta(v,n)    time ship v arrives at port n
tw(v,n)    time duration ship v waiting for at port n
tl(v,n)    time ship v leaves form port n
ts(n,k)    starting time of port n's time slot k
te(n,k)    ending time of port n's time slot k
qi(v,p)    the amount of ship v load at port p
di(v,d)    the amount of ship v discharge at port d
st1(n,k)    stock of port n at the beginning time of slot k
st2(n,k)    stock of port n at the end time of slot k
la1(v,n)    load of ship v at port n before operation
la2(v,n)    load of ship v at port n after operation

wq(v,p,k)  auxiliary variable for bilinear terms
wq1(v,p,k) auxiliary variable for bilinear terms
wd(v,d,k)  auxiliary variable for bilinear terms
wd1(v,d,k) auxiliary variable for bilinear terms

lawp(v,n,p,k)   auxiliary variable for bilinear terms
lawp1(v,n,p,k)  auxiliary variable for bilinear terms
lawd(v,n,d,k)   auxiliary variable for bilinear terms
lawd1(v,n,d,k)  auxiliary variable for bilinear terms

stf(n,k)   stock of port n at the end of period
gamas(n,k) auxiliary variable for reformulation
gamae(n,k) auxiliary variable for reformulation
tals(v,n,k) auxiliary variable for reformulation
tale(v,n,k) auxiliary variable for reformulation
;

variables
TotalCost  total cost of the shipping;

ta.fx(v,'n1')=0;
*ships arrive virtual port n1 at the beginning of the schedule period
ts.fx('n1',k)=0;
*time slots of virtual port n1 starts at the beginning of the schedule period
la2.fx(v,'n1')=0;
*ships load are 0 at the beginning of the schedule period

alias(n,m);
alias(k,kk);
scalar BigM1, BigM2;
BigM1=TimePeriod;

equation
Slot2Ship(v,n)     a slot be assigned to a ship
NomoreShip(n,k)    no more than 1 ship can operate at any port at the same time
TStCon1(v,n,k)     the starting time of a slot be constrained to an operation time
TStCon2(v,n,k)     the starting time of a slot be constrained to an operation time
TEnCon1(v,n,k)     the ending time of a slot be constrained to an operation time
TEnCon2(v,n,k)     the ending time of a slot be constrained to an operation time
SeqSlot(n,k,kk)    the sequence constraint of time slot
NonNegSlot(n,k)    the slot period cannot be negtive
ArrvTime(v,n)      the time ship v arrive port n
OperStTime(v,n)    the time constraint between operation starting and arrival
WaitTime(v,n)      waiting time of ship v at port n
OperTimep(v,p)     operation time of ship v at port p
OperTimed(v,d)     operation time of ship v at port d
OperTimeV1(v,n)     operation time of ship v at virtual ports
OperTimeV2(v,n)     operation time of ship v at virtual ports
WaitUpperBound(v,n) upper bound of ship v's waiting time
LeavTimeN2(v,n)    leaving time of ship v from n8

StockBalap1(p,k)    stock balance of port p at time period k
StockBalad1(d,k)    stock balance of port d at time period k
StockBalap2(p,k)    stock balance of port p at time period k
StockBalad2(d,k)    stock balance of port d at time period k
StockBalap0(p,k)   stock balance of port p at time period k1
StockBalad0(d,k)   stock balance of port d at time period k1
StockBalapf1(p,k)  stock upperbound of port p at the final time period
StockBaladf1(d,k)  stock upperbound of port d at the final time period
StockBalapf2(p,k)  stock lowerbound of port p at the final time period
StockBaladf2(d,k)  stock lowerbound of port d at the final time period
StockUpB1(n,k)      stock upperbound of ports n
StockLoB1(n,k)      stock lowerbound of ports n
StockUpB2(n,k)      stock upperbound of ports n
StockLoB2(n,k)      stock lowerbound of ports n

LoadBalap1(v,p)     load balance of ship v at port p
LoadBalad1(v,d)     load balance of ship v at port d
LoadBalap2(v,p)     load balance of ship v at port p
LoadBalad2(v,d)     load balance of ship v at port d
LoadUpB(v,p)       Upper bound of ship v's load after operation on port p
*LoadLoB(v,n)       Lower bound of ship v's load after operation on port n

WQtrans1(v,p,k)    transition of bilinear terms
WQtrans2(v,p,k)    transition of bilinear terms
WQtrans3(v,p,k)    transition of bilinear terms
WDtrans1(v,d,k)    transition of bilinear terms
WDtrans2(v,d,k)    transition of bilinear terms
WDtrans3(v,d,k)    transition of bilinear terms

lawptrans1(v,m,p,k)    transition of bilinear terms
lawptrans2(v,m,p,k)    transition of bilinear terms
lawptrans3(v,m,p,k)    transition of bilinear terms

lawdtrans1(v,m,d,k)    transition of bilinear terms
lawdtrans2(v,m,d,k)    transition of bilinear terms
lawdtrans3(v,m,d,k)    transition of bilinear terms

OFcost             total cost

Tal2tsRefm(v,n)     reformulation
Tal2teRefm(v,n)     reformulation
Tal2toRefm(n,k)     reformulation
Tal2tlRefm(n,k)     reformulation
UBTals(v,n,k)         reformulation
UBTale(v,n,k)         reformulation
UBGamas(n,k)         reformulation
UBGamae(n,k)         reformulation
Seqy(n,k,kk)         reformulation
;

Slot2Ship(v,n)$(not ori(n))..
        sum(k$kn(n,k),w(v,n,k)) =e= sum(m,x(v,m,n));
*        sum(k$kn('n1',k),w(v,'n1',k)) =e= sum(m,x(v,m,'n1'));
NomoreShip(n,k)$(kn(n,k)and (p(n)or d(n)))..
        sum(v,w(v,n,k))+y(n,k) =l= 1;
TStCon1(v,n,k)$kn(n,k)..
        to(v,n)-ts(n,k) =g= -BigM1*(1-w(v,n,k));
TStCon2(v,n,k)$kn(n,k)..
        to(v,n)-ts(n,k) =l= BigM1*(1-w(v,n,k));
TEnCon1(v,n,k)$kn(n,k)..
        tl(v,n)-te(n,k) =g= -BigM1*(1-w(v,n,k));
TEnCon2(v,n,k)$kn(n,k)..
        tl(v,n)-te(n,k) =l= BigM1*(1-w(v,n,k));
SeqSlot(n,k,kk)$((ord(kk)gt ord(k)) and kn(n,k) and kn(n,kk))..
        ts(n,kk) =g= te(n,k);
NonNegSlot(n,k)..
        te(n,k) =g= ts(n,k);
ArrvTime(v,n)..
        ta(v,n) =e= sum(m,x(v,m,n)*(tl(v,m)+dist(m,n)));
OperStTime(v,n)..
        to(v,n) =g= ta(v,n);
WaitTime(v,n)..
        tw(v,n) =e= to(v,n)-ta(v,n);
OperTimep(v,p)..
        tl(v,p) =e= to(v,p)+sum(n,x(v,n,p)*(tf(v,p)+tv(v,p)*qi(v,p)));
OperTimed(v,d)..
        tl(v,d) =e= to(v,d)+sum(n,x(v,n,d)*(tf(v,d)+tv(v,d)*di(v,d)));
OperTimeV1(v,n)$(des(n))..
        tl(v,n) =e= to(v,n);
OperTimeV2(v,n)$(des(n))..
        to(v,n) =e= ta(v,n);
WaitUpperBound(v,n)$(not ori(n))..
        tw(v,n) =l= TimeWUp(v);
LeavTimeN2(v,'n2')..
        ta(v,'n2') =l= TimePeriod;

StockBalap1(p,k)$(kn(p,k))..
        st1(p,k)-sum(v,wq(v,p,k))+RProd(p)*(te(p,k)-ts(p,k))
           =e=st2(p,k);
StockBalap2(p,k)$((ord(k) ne Cardk(p))and kn(p,k))..
        st2(p,k)+sum(kk$(ord(kk)eq ord(k)+1),RProd(p)*(ts(p,kk)-te(p,k)))
           =e=sum(kk$(ord(kk)eq ord(k)+1),st1(p,kk));

WQtrans1(v,p,k)$kn(p,k)..
        wq(v,p,k) =l= laUp(v)*w(v,p,k);
WQtrans2(v,p,k)$kn(p,k)..
        wq1(v,p,k) =l= laUp(v)*(1-w(v,p,k));
WQtrans3(v,p,k)$kn(p,k)..
        qi(v,p) =e= wq(v,p,k)+wq1(v,p,k);

StockBalad1(d,k)$(kn(d,k))..
        st1(d,k)+sum(v,wd(v,d,k))-RCons(d)*(te(d,k)-ts(d,k))
           =e=st2(d,k);
StockBalad2(d,k)$((ord(k) ne Cardk(d))and kn(d,k))..
        st2(d,k)-sum(kk$(ord(kk)eq ord(k)+1),RCons(d)*(ts(d,kk)-te(d,k)))
           =e=sum(kk$(ord(kk)eq ord(k)+1),st1(d,kk));

WDtrans1(v,d,k)$kn(d,k)..
        wd(v,d,k) =l= laUp(v)*w(v,d,k);
WDtrans2(v,d,k)$kn(d,k)..
        wd1(v,d,k) =l= laUp(v)*(1-w(v,d,k));
WDtrans3(v,d,k)$kn(d,k)..
        di(v,d) =e= wd(v,d,k)+wd1(v,d,k);

StockBalap0(p,'k1')..
        st1(p,'k1') =e= StIni(p)+RProd(p)*ts(p,'k1');
StockBalad0(d,'k1')..
        st1(d,'k1') =e= StIni(d)-RCons(d)*ts(d,'k1');
StockBalapf1(p,k)$((ord(k) eq Cardk(p))and kn(p,k))..
        st2(p,k)+RProd(p)*(TimePeriod-te(p,k))
           =l= StockUp(p);
StockBalapf2(p,k)$((ord(k) eq Cardk(p))and kn(p,k))..
        st2(p,k)+RProd(p)*(TimePeriod-te(p,k))
           =g= StockLo(p);
StockBaladf1(d,k)$((ord(k) eq Cardk(d))and kn(d,k))..
        st2(d,k)-RCons(d)*(TimePeriod-te(d,k))
           =l= StockUp(d);
StockBaladf2(d,k)$((ord(k) eq Cardk(d))and kn(d,k))..
        st2(d,k)-RCons(d)*(TimePeriod-te(d,k))
           =g= StockLo(d);
StockUpB1(n,k)$kn(n,k)..
        st1(n,k) =l= StockUp(n);
StockLoB1(n,k)$kn(n,k)..
        st1(n,k) =g= StockLo(n);
StockUpB2(n,k)$kn(n,k)..
        st2(n,k) =l= StockUp(n);
StockLoB2(n,k)$kn(n,k)..
        st2(n,k) =g= StockLo(n);

LoadBalap1(v,p)..
*        la(v,p) =g= sum(k,w(v,p,k))*(sum(m,x(v,m,p)*la(v,m))+ qi(v,p));
        la2(v,p) =e= sum(k$kn(p,k),wq(v,p,k))+la1(v,p);
LoadBalad1(v,d)..
*        la(v,d) =l= sum(k,w(v,d,k))*(sum(m,x(v,m,d)*la(v,m))- di(v,d));
        la2(v,d) =e= -sum(k$kn(d,k),wd(v,d,k))+la1(v,d);
LoadBalap2(v,p)..
*        la(v,p) =g= sum(k,w(v,p,k))*(sum(m,x(v,m,p)*la(v,m))+ qi(v,p));
        la1(v,p) =e= sum((m,k)$kn(p,k),x(v,m,p)*lawp(v,m,p,k));
LoadBalad2(v,d)..
*        la(v,d) =l= sum(k,w(v,d,k))*(sum(m,x(v,m,d)*la(v,m))- di(v,d));
        la1(v,d) =e= sum((m,k)$kn(d,k),x(v,m,d)*lawd(v,m,d,k));
LoadUpB(v,p)..
        la2(v,p) =l= laUp(v);

lawptrans1(v,m,p,k)$kn(p,k)..
        lawp(v,m,p,k) =l= laUp(v)*w(v,p,k);
lawptrans2(v,m,p,k)$kn(p,k)..
        lawp1(v,m,p,k) =l= laUp(v)*(1-w(v,p,k));
lawptrans3(v,m,p,k)$kn(p,k)..
        la2(v,m) =e= lawp(v,m,p,k)+lawp1(v,m,p,k);

lawdtrans1(v,m,d,k)$kn(d,k)..
        lawd(v,m,d,k) =l= laUp(v)*w(v,d,k);
lawdtrans2(v,m,d,k)$kn(d,k)..
        lawd1(v,m,d,k) =l= laUp(v)*(1-w(v,d,k));
lawdtrans3(v,m,d,k)$kn(d,k)..
        la2(v,m) =e= lawd(v,m,d,k)+lawd1(v,m,d,k);


OFcost..
        TotalCost =e=
                 sum(v,
                    sum((n,m), x(v,n,m)*(Csailf(v)+Csailv(v)*la2(v,n))*dist(n,m))
                  + sum(n,Cwait(v,n)*tw(v,n))
                  + sum(p,Coper(v,p)*(tl(v,p)-to(v,p)))
                  + sum(d,Coper(v,d)*(tl(v,d)-to(v,d)))
                     );

Tal2tsRefm(v,n)$(not ori(n))..
       sum(m$(not des(m)), x(v,m,n))*to(v,n) =e= sum(k$kn(n,k),Tals(v,n,k));
Tal2teRefm(v,n)$(not ori(n))..
       sum(m$(not des(m)), x(v,m,n))*tl(v,n) =e= sum(k$kn(n,k),Tale(v,n,k));
Tal2toRefm(n,k)$kn(n,k)..
       ts(n,k)=e= sum(v,Tals(v,n,k))+Gamas(n,k);
Tal2tlRefm(n,k)$kn(n,k)..
       te(n,k)=e= sum(v,Tale(v,n,k))+Gamae(n,k);
UBTals(v,n,k)$kn(n,k)..
       Tals(v,n,k)=l= w(v,n,k)*BigM1;
UBTale(v,n,k)$kn(n,k)..
       Tale(v,n,k)=l= w(v,n,k)*BigM1;
UBGamas(n,k)$kn(n,k)..
       Gamas(n,k)=l= y(n,k)*BigM1;
UBGamae(n,k)$kn(n,k)..
       Gamae(n,k)=l= y(n,k)*BigM1;
Seqy(n,k,kk)$((ord(kk)eq ord(k)+1)and kn(n,k))..
       y(n,k)=l=y(n,kk);

model SC
/
Slot2Ship
NomoreShip
*TStCon1
*TStCon2
*TEnCon1
*TEnCon2
SeqSlot
NonNegSlot
ArrvTime
OperStTime
WaitTime
OperTimep
OperTimed
OperTimeV1
OperTimeV2
WaitUpperBound
LeavTimeN2
StockBalap1
StockBalad1
StockBalap2
StockBalad2
*$ontext
StockBalap0
StockBalad0
StockBalapf1
StockBaladf1
StockBalapf2
StockBaladf2
StockUpB1
StockLoB1
StockUpB2
StockLoB2
*$offtext
*$ontext
LoadBalap1
LoadBalad1
LoadBalap2
LoadBalad2
LoadUpB
*$offtext
WQtrans1
WQtrans2
WQtrans3
WDtrans1
WDtrans2
WDtrans3
*$ontext
lawptrans1
lawptrans2
lawptrans3

lawdtrans1
lawdtrans2
lawdtrans3
*$offtext
OFcost

Tal2tsRefm
Tal2teRefm
Tal2toRefm
Tal2tlRefm
UBTals
UBTale
UBGamas
UBGamae
Seqy
/;

OPTION SYSOUT = ON;
option limrow = 100;
SC.optcr = 0.000001;

Solve SC using mip minimizing TotalCost;

scalar
costs
costw
costo
;
costs=sum(v,sum((n,m), x(v,n,m)*(Csailf(v)+Csailv(v)*la2.l(v,n))*dist(n,m)));
costw=sum(v,sum(n,Cwait(v,n)*tw.l(v,n)));
costo=sum(v,sum(p,Coper(v,p)*(tl.l(v,p)-to.l(v,p)))
                  + sum(d,Coper(v,d)*(tl.l(v,d)-to.l(v,d)))
                     );

stf.l(p,k)$((ord(k) eq Cardk(p))and kn(p,k))=st2.l(p,k)+RProd(p)*(TimePeriod-te.l(p,k));
stf.l(d,k)$((ord(k) eq Cardk(d))and kn(d,k))=st2.l(d,k)-RCons(d)*(TimePeriod-te.l(d,k));
display
w.l,
to.l,
ta.l,
tw.l,
tl.l,
ts.l,
te.l,
dist,
qi.l,
di.l,
st1.l,
st2.l
stf.l,
la1.l,
la2.l
lawp.l,
lawd.l,

TotalCost.l,
costs,
costw,
costo;
