*---------------------------------------------------------------------------------
*     Create Sets
*---------------------------------------------------------------------------------
set n    ports                 /n1*n8/;
*n7, n8 are virtual original and final ports attached with n1
set p(n) production ports      /n3*n6/;
set d(n) consumption ports     /n7*n8/;
set ori(n) virtual original port /n1/;
set des(n) virtual destination port /n2/;
set v    ships                 /v1*v3/;
set k    time slots for conti  /k1*k10/;
set t    time for discrete     /t1*t140/;
set l    event point           /l1*l100/;

set kn(n,k) time slots for ports;
set kp(p,k) time slots for some production ports;
set kd(d,k) time slots for some consumption ports;

set i iteration index       /1*100/;
alias(k,kk);
parameter Cardk(n) the number of time slots of port n;
$ontext
Cardk(n)=card(v);

loop(kk$(ord(kk) lt card(v)+1),
         kp(p,k)$(ord(kk)eq ord(k))=yes;
         kd(d,k)$(ord(kk)eq ord(k))=yes;
         kn(n,k)$(ord(kk)eq ord(k))=yes;
     );

display n, p, d, v, k, kn, kp, kd;
$offtext
*---------------------------------------------------------------------------------
*     Create parameters
*---------------------------------------------------------------------------------
Scalar TimePeriod;
TimePeriod=70;

parameter CapP(n) capacity of port n
/
n1 1
n2 1
n3 1
n4 1
n5 1
n6 1
n7 1
n8 1
/;



parameter Nep(n)  number of event point of port n
*$ontext
/
n1 9
n2 9
n3 6
n4 6
n5 3
n6 3
n7 3
n8 6
/
*$offtext
;

parameter laUp(v) upperbound of ship v's load
/
v1 600
v2 1200
v3 600
/;

parameter StIni(n) stock of port n at the beginning of the schedule period
/
n1 0
n2 0
n3 350
n4 350
n5 380
n6 350
n7 1450
n8 800
/;



table dist(n,n) distances between ports
      n1    n2    n3    n4    n5    n6    n7    n8
n1    0     0     0     7     14    12    15    20
n2    0     0     0     0     0     0     0     0
n3    0     0     0     7     14    12    15    20
n4    0     7     7     0     16    14    20    25
n5    0     14    14    16    0     17    6     10
n6    0     12    12    20    17    0     8     12
n7    0     15    15    20    6     8     0     10
n8    0     20    20    25    10    12    10    0

;
display StIni, dist;

parameter RProd(p) production rate of production ports
/
n3 15
n4 15
n5 12
n6 13
/;

parameter RCons(d) consumtion rate of consumption ports
/
n7 35
n8 20
/;

display RProd, RCons;

table tf(v,n)  fixed time of the loading or discharding a ship at a port
       n3     n4     n5     n6     n7     n8
v1     0.5    0.5    0.5    0.5    0.5    0.5
v2     0.5    0.5    0.5    0.5    0.5    0.5
v3     0.5    0.5    0.5    0.5    0.5    0.5
;

table tv(v,n)  variable time of the loading or discharding a ship at a port
       n3     n4     n5     n6     n7     n8
v1     0.005  0.005  0.005  0.005  0.005  0.005
v2     0.005  0.005  0.005  0.005  0.005  0.005
v3     0.005  0.005  0.005  0.005  0.005  0.005
;
display tf, tv;

parameter Csailf(v) fixed sailing cost of the ships
/
v1 20
v2 18
v3 18
/;

parameter Csailv(v) unit sailing cost of the ships
/
v1 0.3
v2 0.4
v3 0.3
/;

table Cwait(v,n) unit waiting cost of the ship at a port
     n1    n2    n3    n4    n5    n6    n7    n8
v1   0     10000 12    12    12    12    12    12
v2   0     10000 10    10    10    10    10    10
v3   0     10000 10    10    10    10    10    10
;

table Coper(v,n) unit operation cost of the ship at a port
     n1    n2    n3    n4    n5    n6    n7    n8
v1   10000 10000 22    22    22    22    22    22
v2   10000 10000 20    20    20    20    20    20
v3   10000 10000 20    20    20    20    20    20
;
display Csailf, Csailv, Cwait, Coper;

parameter StockUp(n) upperbound of the ports' stock
/
n3   1100
n4   1300
n5   780
n6   900
n7   1650
n8   950
/;

parameter StockLo(n) lowerbound of the ports' stock
/
n3   10
n4   5
n5   5
n6   5
n7   5
n8   5
/;
display StockUp, StockLo;

parameter TimeWUp(v) upperbound of the ship waiting time
/
v1   7
v2   10
v3   7
/;
display TimeWUp;

table x(v,n,n) routing of the ships
         n1   n2   n3   n4   n5   n6   n7   n8
v1. n1   0    0    1    0    0    0    0    0
v1. n2   0    0    0    0    0    0    0    0
v1. n3   0    0    0    1    0    0    0    0
v1. n4   0    0    0    0    0    0    0    1
v1. n5   0    0    0    0    0    0    0    0
v1. n6   0    0    0    0    0    0    0    0
v1. n7   0    0    0    0    0    0    0    0
v1. n8   0    1    0    0    0    0    0    0

v2. n1   0    0    1    0    0    0    0    0
v2. n2   0    0    0    0    0    0    0    0
v2. n3   0    0    0    0    1    0    0    0
v2. n4   0    0    0    0    0    0    0    0
v2. n5   0    0    0    0    0    0    1    0
v2. n6   0    0    0    0    0    0    0    0
v2. n7   0    1    0    0    0    0    0    0
v2. n8   0    0    0    0    0    0    0    0

v3. n1   0    0    0    1    0    0    0    0
v3. n2   0    0    0    0    0    0    0    0
v3. n3   0    0    0    0    0    0    0    0
v3. n4   0    0    0    0    0    1    0    0
v3. n5   0    0    0    0    0    0    0    0
v3. n6   0    0    0    0    0    0    0    1
v3. n7   0    0    0    0    0    0    0    0
v3. n8   0    1    0    0    0    0    0    0
;
display x;

Parameter Numslot(n),Numslot1(n);
alias(n,nn);
Numslot(n)$ (ori(n))= card(v);
Numslot(n)$ (not ori(n))= sum((v,nn), x(v,nn,n));
*$ontext
Cardk(n)=Numslot(n);
Nep(n)=3*Numslot(n);
loop(kk,
         kp(p,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(p)+1))=yes;
         kd(d,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(d)+1))=yes;
         kn(n,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(n)+1))=yes;
     );

display Numslot, Nep, n, p, d, v, k, kn, kp, kd;
*$offtext
