*---------------------------------------------------------------------------------
*     Create Sets
*---------------------------------------------------------------------------------
set n    ports                 /n1*n12/;
*n7, n8 are virtual original and final ports attached with n1
set p(n) production ports      /n3*n5/;
set d(n) consumption ports     /n6*n12/;
set ori(n) virtual original port /n1/;
set des(n) virtual destination port /n2/;
set v    ships                 /v1*v6/;
set k    time slots for conti  /k1*k10/;
set t    time for discrete     /t1*t160/;
set l    event point           /l1*l100/;

set kn(n,k) time slots for ports;
set kp(p,k) time slots for some production ports;
set kd(d,k) time slots for some consumption ports;

set i iteration index       /1*100/;
alias(k,kk);
alias(n,nn);
parameter Cardk(n) the number of time slots of port n;
$ontext
Cardk(n)=card(v);
loop(kk$(ord(kk) lt card(v)+1),
         kp(p,k)$(ord(kk)eq ord(k))=yes;
         kd(d,k)$(ord(kk)eq ord(k))=yes;
         kn(n,k)$(ord(kk)eq ord(k))=yes;
     );
$offtext
*---------------------------------------------------------------------------------
*     Create parameters
*---------------------------------------------------------------------------------
Scalar TimePeriod;
TimePeriod=80;

parameter CapP(n) capacity of port n
/
n1 1
n2 1
n3 1
n4 1
n5 1
n6 1
n7 1
n8 1
n9 1
n10 1
n11 1
n12 1
/;

parameter Nep(n)  number of event point of port n
/
n1 18
n2 18
n3 9
n4 6
n5 6
n6 3
n7 6
n8 6
n9 6
n10 6
n11 6
n12 6
/;

parameter laUp(v) upperbound of ship v's load
/
v1 660
v2 300
v3 350
v4 800
v5 300
v6 350
/;

parameter StIni(n) stock of port n at the beginning of the schedule period
/
n1 0
n2 0
n3 1050
n4 200
n5 450
n6 350
n7 400
n8 800
n9 400
n10 900
n11 950
n12 1200
/;


parameter StockUp(n) upperbound of the ports' stock
/
n3   19500
n4   16000
n5   17500
n6   15200
n7   17700
n8   19000
n9   17000
n10  15500
n11  14000
n12  14500
/;

parameter StockLo(n) lowerbound of the ports' stock
/
n3   10
n4   5
n5   5
n6   5
n7   5
n8   5
n9   5
n10  5
n11  5
n12  10
/;
display StockUp, StockLo;

table dist(n,n) distances between ports
      n1    n2    n3    n4    n5    n6    n7    n8    n9    n10    n11    n12
n1    0     20    0     7     5     12    9     11    15    14     16     20
n2    20    0     20    15    14    13    10     9    10    6      5      0
n3    0     20    0     7     5     12    9     11    15    14     16     20
n4    7     15    7     0     6     6     7     11    9     10     13     15
n5    5     14    5     6     0     12    6     6     14    11     11     14
n6    12    13    12    6     12    0     8     12    5     8      13     13
n7    9     10    9     7     6     8     0     5     9     5      8      10
n8    11    9     11    11    6     12    5     0     12    8      4      9
n9    15    10    15    9     14    5     9     12    0     5      10     10
n10   14    6     14    10    11    8     5     8     5     0      6      6
n11   16    5     16    13    11    13    8     4     10    6      0      5
n12   20    0     20    15    14    13    10    9     10    6      5      0
;
display StIni, dist;

parameter RProd(p) production rate of production ports
/
n3 40
n4 30
n5 25
/;

parameter RCons(d) consumtion rate of consumption ports
/
n6 5
n7 8
n8 15
n9 12
n10 15
n11 17
n12 23
/;

display RProd, RCons;

table tf(v,n)  fixed time of the loading or discharding a ship at a port
       n3     n4     n5     n6     n7     n8     n9     n10     n11     n12
v1     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
v2     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
v3     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
v4     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
v5     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
v6     0.5    0.5    0.5    0.5    0.5    0.5    0.5    0.5     0.5     0.5
;

table tv(v,n)  variable time of the loading or discharding a ship at a port
       n3     n4     n5     n6     n7     n8      n9     n10     n11     n12
v1     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
v2     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
v3     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
v4     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
v5     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
v6     0.005  0.005  0.005  0.005  0.005  0.005   0.005  0.005   0.005   0.005
;
display tf, tv;

parameter Csailf(v) fixed sailing cost of the ships
/
v1 20
v2 18
v3 15
v4 15
v5 18
v6 20
/;

parameter Csailv(v) unit sailing cost of the ships
/
v1 0.3
v2 0.4
v3 0.4
v4 0.3
v5 0.4
v6 0.4
/;

table Cwait(v,n) unit waiting cost of the ship at a port
     n1    n2    n3    n4    n5    n6    n7    n8    n9    n10   n11   n12
v1   0     10000 12    12    12    12    12    12    12    12    12    12
v2   0     10000 10    10    10    10    10    10    10    10    10    10
v3   0     10000 10    10    10    10    10    10    10    10    10    10
v4   0     10000 12    12    12    12    12    12    12    12    12    12
v5   0     10000 10    10    10    10    10    10    10    10    10    10
v6   0     10000 12    12    12    12    12    12    12    12    12    12
;

table Coper(v,n) unit operation cost of the ship at a port
     n1    n2    n3    n4    n5    n6    n7    n8    n9    n10   n11   n12
v1   10000 10000 22    22    22    22    22    22    22    22    22    22
v2   10000 10000 20    20    20    20    20    20    20    20    20    20
v3   10000 10000 20    20    20    20    20    20    20    20    20    20
v4   10000 10000 22    22    22    22    22    22    22    22    22    22
v5   10000 10000 20    20    20    20    20    20    20    20    20    20
v6   10000 10000 22    22    22    22    22    22    22    22    22    22
;
display Csailf, Csailv, Cwait, Coper;

parameter TimeWUp(v) upperbound of the ship waiting time
/
v1   5
v2   7
v3   3
v4   3
v5   5
v6   4
/;
display TimeWUp;

table x(v,n,n) routing of the ships
         n1   n2   n3   n4   n5   n6   n7   n8   n9   n10   n11   n12
v1. n1   0    0    1    0    0    0    0    0    0    0     0     0
v1. n2   0    0    0    0    0    0    0    0    0    0     0     0
v1. n3   0    0    0    0    0    1    0    0    0    0     0     0
v1. n4   0    0    0    0    0    0    0    0    0    0     0     0
v1. n5   0    0    0    0    0    0    0    0    0    0     0     0
v1. n6   0    0    0    0    0    0    0    0    1    0     0     0
v1. n7   0    0    0    0    0    0    0    0    0    0     0     0
v1. n8   0    0    0    0    0    0    0    0    0    0     0     0
v1. n9   0    0    0    0    0    0    0    0    0    1     0     0
v1. n10  0    0    0    0    0    0    0    0    0    0     0     1
v1. n11  0    0    0    0    0    0    0    0    0    0     0     0
v1. n12  0    1    0    0    0    0    0    0    0    0     0     0

v2. n1   0    0    0    1    0    0    0    0    0    0     0     0
v2. n2   0    0    0    0    0    0    0    0    0    0     0     0
v2. n3   0    0    0    0    0    0    0    0    0    0     0     0
v2. n4   0    0    0    0    0    0    1    0    0    0     0     0
v2. n5   0    0    0    0    0    0    0    0    0    0     0     0
v2. n6   0    0    0    0    0    0    0    0    0    0     0     0
v2. n7   0    0    0    0    0    0    0    0    0    0     1     0
v2. n8   0    0    0    0    0    0    0    0    0    0     0     0
v2. n9   0    0    0    0    0    0    0    0    0    0     0     0
v2. n10  0    0    0    0    0    0    0    0    0    0     0     0
v2. n11  0    1    0    0    0    0    0    0    0    0     0     0
v2. n12  0    0    0    0    0    0    0    0    0    0     0     0

v3. n1   0    0    0    0    1    0    0    0    0    0     0     0
v3. n2   0    0    0    0    0    0    0    0    0    0     0     0
v3. n3   0    0    0    0    0    0    0    0    0    0     0     0
v3. n4   0    0    0    0    0    0    0    0    0    0     0     0
v3. n5   0    0    0    0    0    0    0    1    0    0     0     0
v3. n6   0    0    0    0    0    0    0    0    0    0     0     0
v3. n7   0    0    0    0    0    0    0    0    0    0     0     0
v3. n8   0    1    0    0    0    0    0    0    0    0     0     0
v3. n9   0    0    0    0    0    0    0    0    0    0     0     0
v3. n10  0    0    0    0    0    0    0    0    0    0     0     0
v3. n11  0    0    0    0    0    0    0    0    0    0     0     0
v3. n12  0    0    0    0    0    0    0    0    0    0     0     0

v4. n1   0    0    1    0    0    0    0    0    0    0     0     0
v4. n2   0    0    0    0    0    0    0    0    0    0     0     0
v4. n3   0    0    0    0    0    0    1    0    0    0     0     0
v4. n4   0    0    0    0    0    0    0    0    0    0     0     0
v4. n5   0    0    0    0    0    0    0    0    0    0     0     0
v4. n6   0    0    0    0    0    0    0    0    0    0     0     0
v4. n7   0    0    0    0    0    0    0    0    0    0     0     1
v4. n8   0    0    0    0    0    0    0    0    0    0     0     0
v4. n9   0    0    0    0    0    0    0    0    0    0     0     0
v4. n10  0    0    0    0    0    0    0    0    0    0     0     0
v4. n11  0    0    0    0    0    0    0    0    0    0     0     0
v4. n12  0    1    0    0    0    0    0    0    0    0     0     0

v5. n1   0    0    0    1    0    0    0    0    0    0     0     0
v5. n2   0    0    0    0    0    0    0    0    0    0     0     0
v5. n3   0    0    0    0    0    0    0    0    0    0     0     0
v5. n4   0    0    0    0    0    0    0    0    1    0     0     0
v5. n5   0    0    0    0    0    0    0    0    0    0     0     0
v5. n6   0    0    0    0    0    0    0    0    0    0     0     0
v5. n7   0    0    0    0    0    0    0    0    0    0     0     0
v5. n8   0    0    0    0    0    0    0    0    0    0     0     0
v5. n9   0    0    0    0    0    0    0    0    0    0     1     0
v5. n10  0    0    0    0    0    0    0    0    0    0     0     0
v5. n11  0    1    0    0    0    0    0    0    0    0     0     0
v5. n12  0    0    0    0    0    0    0    0    0    0     0     0

v6. n1   0    0    1    0    0    0    0    0    0    0     0     0
v6. n2   0    0    0    0    0    0    0    0    0    0     0     0
v6. n3   0    0    0    0    1    0    0    0    0    0     0     0
v6. n4   0    0    0    0    0    0    0    0    0    0     0     0
v6. n5   0    0    0    0    0    0    0    1    0    0     0     0
v6. n6   0    0    0    0    0    0    0    0    0    0     0     0
v6. n7   0    0    0    0    0    0    0    0    0    0     0     0
v6. n8   0    0    0    0    0    0    0    0    0    1     0     0
v6. n9   0    0    0    0    0    0    0    0    0    0     0     0
v6. n10  0    1    0    0    0    0    0    0    0    0     0     0
v6. n11  0    0    0    0    0    0    0    0    0    0     0     0
v6. n12  0    0    0    0    0    0    0    0    0    0     0     0




;
display x;

Parameter Numslot(n),Numslot1(n);

Numslot(n)$ (ori(n))= card(v);
Numslot(n)$ (not ori(n))= sum((v,nn), x(v,nn,n));
Cardk(n)=Numslot(n);
*$ontext
Nep(n)=3*Numslot(n);

loop(kk,
         kp(p,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(p)+1))=yes;
         kd(d,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(d)+1))=yes;
         kn(n,k)$((ord(kk)eq ord(k))and (ord(kk) lt Numslot(n)+1))=yes;
     );

display Numslot, Nep, n, p, d, v, k, kn, kp, kd;
*$offtext
